<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 27/04/17
 * Time: 15:47
 */

namespace Controller;

use Exception\CustomException;
use Util\Utilities;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Operador extends \Generated\Controller\Operador{

    /**
     * @var \Service\Operador
     */
    private $operadorService;

    public function __construct(Manager $dataSourceManager,\Service\Operador $operadorService) {
        parent::__construct($dataSourceManager);
        $this->operadorService = $operadorService;
    }

    public function findByCurrentCdcliente(Request\Filter $request, Response $response){
        try{
            $page      = $request->getFilterCriteria()->getPage();
            $pageSize  = $request->getFilterCriteria()->getPageSize();
            $condition = $request->getFilterCriteria()->getConditions();
            $dataSet   = $this->operadorService->findAllOperadorRelationshipCliente($condition, $page, $pageSize);
            $response->addDataSet(new DataSet('findByCurrentCdcliente', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findAllOperador(Request\Filter $request, Response $response){
        try{
            $request->getFilterCriteria()->addCondition("IDATIVO", DataSource\FilterCriteria::EQ, "S");
            $condition = $request->getFilterCriteria()->getConditions();
            $dataSet = $this->operadorService->findAllOperador($condition);
            $response->addDataSet(new DataSet('findAllOperador', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findAllOperadorRelationshipCliente(Request\Filter $request, Response $response){
        try{
            $request->getFilterCriteria()->addCondition("IDATIVO", DataSource\FilterCriteria::EQ, "S");
            $condition = $request->getFilterCriteria()->getConditions();
            $dataSet = $this->operadorService->findAllOperadorRelationshipCliente($condition);
            $response->addDataSet(new DataSet('findAllOperadorRelationshipCliente', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findAllOperadorResponsible(Request\Filter $request, Response $response){
        try{
            $page      = $request->getFilterCriteria()->getPage();
            $pageSize  = $request->getFilterCriteria()->getPageSize();
            $condition = $request->getFilterCriteria()->getConditions();
            $dataSet = $this->operadorService->findAllOperadorResponsible($condition, $page, $pageSize);
            $response->addDataSet(new DataSet('findAllOperadorResponsible', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findAvailableCliente(Request\Filter $request, Response $response){
        try{
            $condition = $request->getFilterCriteria()->getConditions();
            $dataSet = $this->operadorService->findAvailableCliente($condition);
            $response->addDataSet(new DataSet('findAvailableCliente', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findAllAvailableCliente(Request\Filter $request, Response $response){
        try{
            $condition = $request->getFilterCriteria()->getConditions();
            $dataSet = $this->operadorService->findAllAvailableCliente($condition);
            $response->addDataSet(new DataSet('findAllAvailableCliente', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function currentCliente(Request\Filter $request, Response $response){
        try{
            $dataSet = $this->operadorService->currentCliente();
            $response->addDataSet(new DataSet('currentCliente', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findProfileByOperador(Request\Filter $request, Response $response){
        try{
            if(Utilities::currentUserIsTeknisa()){
                $this->dataSourceName = 'findProfileByOperadorIsTeknisa';
            }else{
                $this->dataSourceName = 'findProfileByOperador';
            }
            parent::find($request, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findOperador(Request\Filter $request, Response $response){
        try{
            if(Utilities::currentUserIsTeknisa()){
                $this->dataSourceName = 'findProfileByOperadorIsTeknisa';
            }else{
                $this->dataSourceName = 'findProfileByOperador';
            }
            parent::find($request, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findProfileForAddProfile(Request\Filter $request, Response $response){
        try{
            $condition = $request->getFilterCriteria()->getConditions();
            $cdoperador = Utilities::getValueInFilterDataSourceByName($condition,'CDOPERADOR');
            $dataSet= array();
            if($cdoperador){
                $dataSet = $this->operadorService->findProfileForAddProfile($cdoperador);
            }
            $response->addDataSet(new DataSet('findProfileForAddProfile', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveWithClientLogged(Request\DataSet $request, Response $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $cdoperador = $row['CDOPERADOR'];
            $nmoperador = $row['NMOPERADOR'];
            $idativo = $row['IDATIVO'];
            $password = empty($row['NEW_PASSWORD']) ? null : $row['NEW_PASSWORD'];
            $dsemailoper = $row['DSEMAILOPER'];
            $cdoperador = $this->operadorService->saveWithClientLogged($cdoperador,$nmoperador,$dsemailoper,$idativo,$password);
            $response->addDataSet(new DataSet('findAllOperador', array(array('CDOPERADOR' => $cdoperador))));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveCurrentCliente(Request\DataSet $request, Response $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $cdcliente = $row['CDCLIENTE'];
            $cliente = $this->operadorService->saveCurrentCliente($cdcliente);
            $response->addDataSet(new DataSet('currentCliente', $cliente));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function removeProfileForAddProfile(Request\DataSet $request, Response $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $nrperfacessparc = $row['NRPERFACESSOPER'];
            $this->operadorService->removeProfileForAddProfile($nrperfacessparc);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveProfileForAddProfile(Request\DataSet $request, Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $profiles = $row['ADD_PROFILES'];
            $cdoperador = $row['CDOPERADOR'];
            $this->operadorService->saveProfileForAddProfile($cdoperador, $profiles);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function changePassword(Request\DataSet $request, Response $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $cdoperador = $row['OPERADOR'];
            $password = $row['NEW_PASSWORD'];
            $cdoperador = $this->operadorService->changePassword($cdoperador,$password);
            $response->addDataSet(new DataSet($request->getDataSet()->getDataSourceName(), array(array('CDOPERADOR' => $cdoperador))));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

} 