<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Util\DateUtil;
use \Teknisa\Model\Geral\Entities\Operador;
use Util\Utilities;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Profile\Entities\ZhUser;


class Login {

    /** @var EntityManager $entityManager */
    protected  $entityManager;
    /** @var \Util\Utilities */
    protected  $environment;
    /** @var \Teknisa\Model\Geral\Repositories\Operador  */
    protected $operadorRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Cliente  */
    protected $clienteRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Paramgeral  */
    protected $paramGeralRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\Funcionario  */
    protected $funcionarioRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\Equipetrab  */
    protected $equipeTrabRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Indice  */
    protected $indiceRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Setor  */
    protected $setorRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource  */
    protected $resourceRepository;
    protected $cdClienteBase;
    /** @var \Teknisa\Libs\Service\Login */
    private $libLoginService;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment,
                                \Teknisa\Libs\Service\Login $libLoginService, $cdClienteBase){
        $this->entityManager = $entityManager;
        $this->environment = $environment;

        $this->operadorRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador');
        $this->clienteRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Cliente');
        $this->indiceRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Indice');
        $this->setorRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Setor');
        $this->paramGeralRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Paramgeral');
        $this->filialRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Filial');
        $this->funcionarioRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Funcionario');
        $this->equipeTrabRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Equipetrab');
        $this->resourceRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        $this->cdClienteBase = $cdClienteBase;
        $this->libLoginService = $libLoginService;
    }

    /**
     * @param \Teknisa\Model\Geral\Entities\Operador $operador
     * @return string
     * @throws \Exception(Messages::ERR_LOGIN_NOT_POSSIBLE)
     */
    private function getASPSession($operador)
    {

        try {
            $result = '';
            if ($operador) {
                /** @var $cliente  \Teknisa\Model\Geral\Entities\Cliente */
                $cliente = $this->clienteRepository->findOneBy(array('cdcliente' => $operador->getCdcliente()));

                $data = DateUtil::adicionaIntervalo(DateUtil::getDataAtual(), $cliente->getNrdiavalpers());
                $dataValidSomaPer = $data->format('d-m-Y');
                $dataValidSomaPerYMD = $data->format('Ymd');

                if ($cliente->getDtvalarqpers() != null) {
                    $dataValidadePer = $cliente->getDtvalarqpers()->format('d-m-Y');
                    $dataValidadePerYMD = $cliente->getDtvalarqpers()->format('Ymd');
                } else {
                    $dataValidadePer = '30/12/1899';
                    $dataValidadePerYMD = '18991230';
                }

                /**@var $paramGeral \Teknisa\Model\Geral\Entities\Paramgeral */
                $paramGeral = $this->paramGeralRepository->findOneBy(array());

                /**@var $filial \Teknisa\Model\Geral\Entities\Filial */
                $filial = $this->filialRepository->findOneBy(array('cdfilial' => $paramGeral->getCdfilial()));

                $totDiaVal = '0';
                if ($operador->getDtultatusenh() != null && $paramGeral->getNrdiavalsenh() != null) {
                    $totDiaVal = DateUtil::adicionaIntervalo($operador->getDtultatusenh(), $paramGeral->getNrdiavalsenh());
                    $totDiaVal = DateUtil::getQtdeDiasEntre($totDiaVal, DateUtil::getDataAtual());
                }

                $cdmoeda = $this->getIndiceByClient($cliente);

                /** @var $funcionario \Teknisa\Model\Teknisa\Entities\Funcionario */
                $funcionario = $this->funcionarioRepository->findOneBy(array(
                    'cdempresa' => $operador->getCdempresa(),
                    'cdfunc' => $operador->getCdfunc()
                ));

                $grupoAltPonto = $this->operadorRepository->verificaGrupoAlteraPonto($operador->getCdoperador());
                $grupodpap = $grupoAltPonto[0]['GRUPODPAP'];

                $nmsetor = '';
                $cdequipetrab = '';
                $nmequipetrab = '';
                $cdoperresp = '';
                $cdtpadmissao = '';
                $cdfunc = $operador->getCdoperador();

                if($funcionario){
                    $cdtpadmissao = $funcionario->getCdtpadmissao();

                    /** @var $equipetrab \Teknisa\Model\Teknisa\Entities\Equipetrab */
                    $equipetrab = $this->equipeTrabRepository->findOneBy(array('cdequipetrab' => $funcionario->getCdequipetrab()));
                    $cdequipetrab = $equipetrab->getCdequipetrab();
                    $nmequipetrab = $equipetrab->getNmequipetrab();
                    $cdoperresp = $equipetrab->getCdoperresp();

                    $setor = $this->getSetorByOperador($operador);
                    $nmsetor = $setor->getNmsetor();
                    $cdfunc =$operador->getCdfunc();

                }


                $result = $operador->getCdoperador() . '@@' .
                    $operador->getNmoperador() . '@@' .
                    $operador->getCdcliente() . '@@' .
                    $cliente->getNmrazsocclie() . '@@' .
                    $cliente->getNrdiavalpers() . '@@' .
                    $dataValidSomaPer . '@@' .
                    $dataValidSomaPerYMD . '@@' .
                    $dataValidadePer . '@@' .
                    $dataValidadePerYMD . '@@' .
                    $cliente->getIdlibgerper() . '@@' .
                    $operador->getIdopermaster() . '@@' .
                    $operador->getIdoperacesso() . '@@' .
                    $cdfunc . '@@' .
                    $totDiaVal . '@@' .
                    $cliente->getNrinsjurclie() . '@@' .
                    $cliente->getIdtrabclie() . '@@' .
                    'N' . '@@' .
                    $cdmoeda . '@@' .
                    $operador->getIdtpoperador() . '@@' .
                    $cliente->getCdativcli() . '@@' .
                    $paramGeral->getCdfilial() . '@@' .
                    $filial->getCdempresa() . '@@' .
                    $cliente->getCdclieprincipal() . '@@' .
                    $operador->getCdsetor() . '@@' .
                    $nmsetor. '@@' .
                    $cdequipetrab . '@@' .
                    $nmequipetrab . '@@' .
                    $cdoperresp . '@@' .
                    $cdtpadmissao . '@@' .
                    $cliente->getNmbdcliente() . '@@' .
                    $grupodpap;
            }
            return $result;
        } catch (\Exception $e) {
            throw new CustomException('ERR_LOGIN_NOT_POSSIBLE', 500, array( 'error' => $e->getMessage()));
        }
    }

    private function isSessionStarted() {
        return $this->environment->getSession()->isStarted();
    }

    public function login($userId, $cdcliente) {
        /** @var \Teknisa\Model\Geral\Entities\Operador $operador */
        $operador = $this->operadorRepository->findOneBy(array('cdoperador' => $userId));
        if($cdcliente) {
            $validateCliente = $this->operadorRepository->findIfCurrenctClienteStillAvaiable($userId, $cdcliente);
            if(empty($validateCliente)) {
                $cdcliente = null;
            }
        }
        if($operador instanceof Operador === false ){
            $operador = null;
            $aspSession = null;
            throw new CustomException('ERR_LOGIN_NOT_POSSIBLE_OPERATOR_NOT_FOUND', 500, array( 'userId' => $userId));
        } else {
            $aspSession = $this->getASPSession($operador);
        }
        if (!$cdcliente) {
            $this->environment->getSession()->set('CDCLIENTE', $operador->getCdcliente());
            $cdcliente = $operador->getCdcliente();
        }
        $this->environment->setInSession("ASPSession", $aspSession);
        $this->environment->setCdClienteCurrent($cdcliente);
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
        $this->setPmlogin($operador);

        return $this->getUserSession();
    }

    public function clearSession() {
        $this->environment->removeFromSession('userId');
        $this->environment->removeFromSession('organizationId');
        $this->environment->removeFromSession('ASPSession');
    }

    public function destroySession() {
        if ($this->isSessionStarted()) {
            $this->clearSession();
            $this->environment->getSession()->destroy();
        }
    }

    public function validateChangeCliente(){
        $cdCliente = $this->environment->getCurrentCdCliente();
        $userId = $this->environment->getCurrentUserId();
        return(!!count($this->operadorRepository->findAvailableCliente($userId,$cdCliente)));
    }

    public function getUserSession()
    {
        $result = array();
        $error = '';
        /** @var Operador $operador */
        if($operador = $this->environment->getCurrentUser()){
            $cdcliente = $this->environment->getCurrentCdCliente();
            /** @var \Teknisa\Model\Geral\Entities\Cliente $cliente */
            $cliente = $this->clienteRepository->findOneBy(array('cdcliente' => $cdcliente));
            $organizacao = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrparcnegocio' => $cliente->getNrparcnegocio()));
            if(!$organizacao){
                if($operador->getCdcliente() != $cdcliente){
                    $cdcliente = $operador->getCdcliente();
                    $this->environment->setCdClienteCurrent($operador->getCdcliente());
                    $cliente = $this->clienteRepository->findOneBy(array('cdcliente' =>$cdcliente));
                    $organizacao = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrparcnegocio' => $cliente->getNrparcnegocio()));
                } else {
                    throw new CustomException('ERR_INVALID_ORGANIZATION_IN_LOGIN', 500, array( 'nmrazsocclie' => $cliente->getNmrazsocclie()));
                }
            }
            $paramOrg = InstanceProvider::getRepository()->getOmParamOrg()->findOneBy(array('nrorg' => $organizacao->getNrorg()));
            $allowProductCall = $paramOrg ? $paramOrg->getIdallowproductcall() : 'N';
            
            if(Utilities::getCdClienteBase() != $cdcliente){
                $accessProfiles = \Zeedhi\DevOps\Util\Utilities::getArrayFromArrayAssociative($this->operadorRepository->getPerfAcessParcByNrorgAndCdoperador($operador->getNrorg(), $operador->getCdoperador()));
            }else{
                $accessProfiles = InstanceProvider::getService()->getLibUserDataRepository()->getPerfAcessParcByNrorgAndNrParcNegocio($operador->getNrorg(), $operador->getNrparcnegocio());
            }
            $result = array(
                'name'  => $operador->getNmoperador(),
                'changeCliente'  => $this->validateChangeCliente(),
                'error'  => $error,
                'login' => array(
                    'name' => $operador->getNmoperador(),
                    'isTeknisaUser' => Utilities::currentUserIsTeknisa(),
                    'activeTicket' => Utilities::getActiveTicket(),
                    'email' => $operador->getDsemailoper(),
                    'operador' => $operador->getCdoperador(),
                    'acesso' => $operador->getNmacesso(),
                    'org' => $operador->getNrorg(),
                    'aspSession' => $this->environment->getInSession('ASPSession'),
                    'clienteName' => $cliente->getNmfantclie(),
                    'clienteNrparcnegocio' => $cliente->getNrparcnegocio(),
                    'clienteNrorg' => $organizacao->getNrorg(),
                    'clienteRS' => $cliente->getNmrazsocclie(),
                    'clienteCd' => $cdcliente,
                    'clienteNmbd' => $cliente->getNmbdcliente(),
                    'allowProductCall' => $allowProductCall,
                    'accessProfiles' => $accessProfiles,
                    'isTeknisaClient' => Utilities::currentClienteIsTeknisa()
                )
            );
        }
        return $result;
    }

    /**
     * @param \Teknisa\Model\Geral\Entities\Cliente $cliente
     * @return  null |\Teknisa\Model\Geral\Entities\Indice
     */
    private function getIndiceByClient(\Teknisa\Model\Geral\Entities\Cliente $cliente = null)
    {
        $cdmoeda = 'real';

        if ($cliente->getCdindice() != null) {
            $id = array(
                "cdtpindice" => $cliente->getCdtpindice(),
                "idpervalor" => $cliente->getIdpervalor(),
                "cdindice" => $cliente->getCdindice(),
                "nrorg" => $cliente->getNrorg(),
            );
            $cdmoeda = $this->indiceRepository->findOneBy($id)->getNmindice();

        }
        return $cdmoeda;
    }

    /**
     * @param \Teknisa\Model\Geral\Entities\Operador  $operador
     * @return null |\Teknisa\Model\Geral\Entities\Setor
     */
    private function getSetorByOperador(\Teknisa\Model\Geral\Entities\Operador  $operador = null)
    {
        $setor = null;
        if ($operador->getCdsetor() != null) {
            $id = array(
                "cdsetor" => $operador->getCdsetor(),
                "nrorg" => $operador->getNrorg(),
            );
            $setor = $this->setorRepository->findOneBy($id);
        }
        return $setor;
    }

    /**
     * @param $operador
     */
    public function setPmlogin(\Teknisa\Model\Geral\Entities\Operador $operador)
    {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
        $resource = $this->resourceRepository->findOneBy(array('externalCode' => $operador->getCdoperador()));
        $pmUserId = 1;
        if ($resource) {
            $pmUserId = $resource->getId();
        }
        $this->createDevOpsZhUser($pmUserId);
        $this->environment->setPMCurrentUserId($pmUserId);
    }

    public function createDevOpsZhUser($resourceId){

        $resourceRepository = InstanceProvider::getRepository()->getResource();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
        $resource = $resourceRepository->find($resourceId);
        $externalId = $resource->getExternalId();
        $userName = $resource->getName();
        $userEmail = $resource->getEmail();

        $zhUserRepository = InstanceProvider::getRepository()->getZhUser();

        $devOpsUser = $zhUserRepository->findOneByExternalResId($resourceId);

        if(is_null($devOpsUser)) {
            $devOpsUser = new ZhUser();
            $devOpsUser->setName($userName);
            $devOpsUser->setEmail($userEmail);
            $devOpsUser->setExternalId($externalId);
            $devOpsUser->setExternalResId($resourceId);
            $this->environment->setDoCurrentUserId(1);
            $this->environment->setDoOrganizationId($resource->getOrganization());
            $this->entityManager->persist($devOpsUser);
            $this->entityManager->flush();
        }

    }
}
