var engine = function (templateManager, eventAggregator, ScreenService, ZHPromise, LibLoginController, MessageService,
                       LibUtilities, localStorageEngine, requestDataSourceEngine, ZhuAclController, ApplicationContext) {

    const BEFORE_PROJECT_STARTUP_EVENT = "beforeProjectStartup";
    const ON_CONTAINER_OPEN_EVENT      = "onContainerOpen";

    eventAggregator.subscribe(ON_CONTAINER_OPEN_EVENT, function (openEvent) {
        var container = openEvent.container;
        if(container && container.label) {
            var updatedTitle = ScreenService.i18n(container.label) + ' - CP';
            templateManager.updateTitle(updatedTitle);
        }
    });

    eventAggregator.subscribe(BEFORE_PROJECT_STARTUP_EVENT, function () {
        if(typeof memoryStorage !== 'undefined') {
            localStorageEngine.getLocalVar = memoryStorage.getLocalVar;
            localStorageEngine.setLocalVar = memoryStorage.setLocalVar;
            setLocalVar = memoryStorage.setLocalVar;
            getLocalVar = memoryStorage.getLocalVar;
        }
        var userId = 'CP_' + localStorage.getItem('operador');
        if(ApplicationContext.ZhPreferences) {
            ApplicationContext.ZhPreferences.setUserID(userId);
        } else {
            ApplicationContext.zhCustomizationsService.registerUser(userId);
        }
        var loginDefer = ZHPromise.defer();
        localStorageEngine.setLocalVar('loginPromise', loginDefer.promise);

        templateManager.project.elementListDefaultRule = 'A';
        LibLoginController.authenticate()
            .then(function(response) {
                CPlogin(response, loginDefer);
            }).catch(function (error) {
                if (error.data) {
                    ScreenService.showMessage(error.data.error, 'ERROR');
                } else {
                    ScreenService.showMessage(JSON.stringify(error), 'ERROR');
                }
            });
    });

    var setLocalVars = function (response, result) {
        localStorageEngine.setLocalVar('CDOPERADOR', response.cdoperador);
        localStorageEngine.setLocalVar('NMOPERADOR', response.nmoperador);
        localStorageEngine.setLocalVar('NRORG', response.nrorg);
        localStorageEngine.setLocalVar('CURRENT_NRORG', response.nrorg);
        localStorageEngine.setLocalVar('NMORG', response.nmorg);
        localStorageEngine.setLocalVar('NMRAZSOCIALORG', response.nmrazsocialorg);
        localStorageEngine.setLocalVar('IDIDIOMA', response.ididioma);
        localStorageEngine.setLocalVar('PROJECT_ID', window.projectId);
        localStorageEngine.setLocalVar('CLIENTE_NRPARCNEGOCIO', result.dataset.login[0].login.clienteNrparcnegocio);
        localStorageEngine.setLocalVar('CLIENTE_NRORG', result.dataset.login[0].login.clienteNrorg);
        localStorageEngine.setLocalVar('CDCLIENTE', result.dataset.login[0].login.clienteCd);
       
    };

    var setAcl = function(libResponse, cpResponse) {
        ZhuAclController.setUserRules(cpResponse.accessProfiles, window.projectId, libResponse.USER_ID, libResponse.ORGANIZATION_ID,
            {
                canConfigureAcl: libResponse.ACL_ADMINISTRATOR,
                rules          : libResponse.ACL_ADMINISTRATOR ? [] : false,
                defaultRule    : libResponse.ACL_ADMINISTRATOR ? 'A' : false
            }
        );
    };

    var prepareLoginData = function(libResponse, data) {
        var login = data.dataset.login[0];
        localStorageEngine.setLocalVar('customerPortalLogin', login.login);
        localStorage.setItem('operador', login.login.operador);
        var userId = 'CP_' + login.login.operador;
        if(ApplicationContext.ZhPreferences) {
            ApplicationContext.ZhPreferences.setUserID(userId);
        } else {
            ApplicationContext.zhCustomizationsService.registerUser(userId);
        }

        var defaultImage = window.isCustomerPortal ? loginConfig.defaultImageProduct : loginConfig.defaultImageModules;
        ScreenService.buildUserData(login.name, [login.login.clienteName], function() {}, defaultImage);
        ScreenService.setHeaderInfo(login.login.clienteName);

        if(!login.changeCliente){
            templateManager.project.userDataActions = [];
        }

        setAcl(libResponse, login.login);
    };

    var CPlogin = function(response, loginDefer){
        localStorageEngine.setLocalVar('ACL_ADMINISTRATOR', response.ACL_ADMINISTRATOR);
        var cdClienteLogin = localStorage.getItem('cdClienteLogin') || null ;
        login(cdClienteLogin, response.USER_ID).then(function(result) {
            if(!result.dataset.login[0].login.isTeknisaUser){
                response.ACCESS_PROFILE = result.dataset.login[0].login.accessProfiles;
            }
            localStorageEngine.setLocalVar('operatorIsTeknisa', result.dataset.login[0].login.isTeknisaUser);
            validateLoginResponse(response, result);
            prepareLoginData(response, result);
            loginDefer.resolve();
            return ZHPromise.when();
        }).catch(function(e) {
            var error = JSON.parse(e.data.error);
            MessageService.getMessageByCode('product#' + error.CODE).then(function(message) {
                ScreenService.showCustomDialog(message, 'CONFIRMATION',null,[
                    {
                        "label": "OK",
                        "code": function () {
                            if (error.CODE === "ERR_INVALID_ORGANIZATION_IN_LOGIN")
                            {
                                LibLoginController.logout();
                            }
                        }
                    }
                ], error.PARAMS);
            });
        });

    };

    var showErrorMessage = function(response) {
        MessageService.getMessageByCode('product#UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    MessageService.getMessageByCode('product#' + messageCode).then(function(message) {
                        ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    ScreenService.showMessage(message, 'ERROR');
                }
            } else if (response && response.message) {
                ScreenService.showMessage(response.message, 'ERROR');
            } else {
                ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    var isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };

    var validateLoginResponse = function (response, result) {
        var defaultResponse = response;
        response = LibUtilities.changeLoginResponseToOldParams(response);
        if(response && response.cdoperador) {
            eventAggregator.publish('loginSuccess', defaultResponse);
            setLocalVars(response, result);
        }
    };
    
    eventAggregator.subscribe('menuBuilded', function (loginResponse) {
        
        var operatorIsTeknisa = localStorageEngine.getLocalVar("operatorIsTeknisa");
        
        if(!operatorIsTeknisa){
            var menuDeploysByClient = templateManager.project.getMenu("deploysByClient");
            if(menuDeploysByClient){
                menuDeploysByClient.isVisible = false;
            }
        }
        
    });

    this.logout = function () {
        MessageService.confirmMessageByCode('product#ALERT_EXIT_SYSTEM', null,
            {
                type: 'CONFIRMATION',
                success: LibLoginController.logout,
                fail: function () {}
            }
        );
    };
    
    var login = function(cdCliente, userId) {
        var filters = [
            {name: "CDCLIENTE", value: cdCliente, operator: "="},
            {name: "USER_ID", value: userId, operator: "="}
        ];
        return requestDataSourceEngine.getDataSource('product#/login', filters);
    };
};

Configuration(function (ContextRegister) {
    ContextRegister.register('engine', engine);
});

function ClienteService(requestDataSourceEngine, LibLoginController) {

    this.loginUpdateSession = function(cdcliente){
        return LibLoginController.updateSession(null, null, { CDCLIENTE: cdcliente }).then(function() {
            var filters = [
                {name: "CDCLIENTE", value: cdcliente, operator: "="}
            ];
            return requestDataSourceEngine.getDataSource('product#/updateCustomerInSession', filters);
        });
    };

    this.getWidgetClienteJson = function () {
        return {
            "name": "CHANGE_CLIENTE",
            "label": "Trocar de cliente",
            "isVisible": true,
            "showAsModal": true,
            "hasPagination": false,
            "searchWithPagination": false,
            "template": "widget/form.html",
            "order": "NMFANTCLIE",
            "showDescriptionOrder": false,
            "actions": [
                {
                    "name": "CANCEL",
                    "label": "Cancelar",
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ClienteController.changeClienteActionEventCancel(args)"
                        }
                    ]
                },
                {
                    "name": "SAVE",
                    "label": "Trocar",
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ClienteController.changeClienteActionEventSave(args)"
                        }
                    ]
                }
            ],
            "events": [],
            "dataSource": {
                "name": "product#/currentCliente",
                "rest": true,
                "lazyLoad": true,
                "localStorage": false,
                "data": []
            },
            "fields": [
                {
                    "name": "NMFANTCLIE",
                    "label": "Cliente",
                    "isActive": true,
                    "focused": true,
                    "isVisible": true,
                    "hasPagination": false,
                    "searchWithPagination": false,
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": ["NMFANTCLIE"],
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "ClienteController.changeClienteFieldOnChangeCliente(args)"
                        }
                    ],
                    "valueField": "CDCLIENTE",
                    "descriptionField": "NMFANTCLIE",
                    "descriptionFieldLabel": "Nome fantasia",
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            "name": "NMFANTCLIE",
                            "label": "Nome Fantasia",
                            "showOnList": true
                        },
                        {
                            "name": "NMRAZSOCCLIE",
                            "label": "Razão Social",
                            "showOnList": true
                        }
                    ],
                    "outData": {
                        "NRINSJURCLIE": "NRINSJURCLIE",
                        "NMFANTCLIE": "NMFANTCLIE",
                        "NMRAZSOCCLIE": "NMRAZSOCCLIE",
                        "CDCLIENTE": "CDCLIENTE"
                    },
                    "dataSource": {
                        "name": "product#/findAvailableCliente",
                        "rest": true,
                        "localStorage": false,
                        "data": []
                    }
                },
                {
                    "name": "NRINSJURCLIE",
                    "label": "Número da inscrição jurídica",
                    "template": "field/text-edit.html",
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "NMFANTCLIE",
                    "label": "Nome Fantasia",
                    "template": "field/text-edit.html",
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "NMRAZSOCCLIE",
                    "label": "Razão Social",
                    "template": "field/text-edit.html",
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                }
            ]
        };
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ClienteService', ClienteService);
});

function ClienteController( ScreenService, templateManager,ClienteService,metaDataFactory, $rootScope, MessageService, ZhFilterCreateFilterConditionsController) {

    var self = this;

    var widgetChangeCliente = {};

    var getFieldClienteFromChangeCliente = function () {
        return widgetChangeCliente.getField('NMFANTCLIE');
    };

    var maskFormatFunction = function (value) {
        if (value) {
            var length = value.length;
            if (length === 11)
                value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
            else if (length === 14)
                value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
        }
        return value;
    };

    this.changeCliente = function(){
        var widget = metaDataFactory.widgetFactory(ClienteService.getWidgetClienteJson(), templateManager.container);
        widgetChangeCliente = widget;
        widget.reload().then(function(){
                widgetChangeCliente.moveToFirst();
                widgetChangeCliente.currentRow.NRINSJURCLIE = maskFormatFunction(widgetChangeCliente.currentRow.NRINSJURCLIE);
            }
        );
        ScreenService.openPopup(widget);
    };

    this.changeClienteActionEventCancel = function(){
        ScreenService.closePopup();
    };

    this.changeClienteFieldOnChangeCliente = function(){
        widgetChangeCliente.currentRow.NRINSJURCLIE = maskFormatFunction(widgetChangeCliente.currentRow.NRINSJURCLIE);
    };

    var showErrorMessage = function(response) {
        MessageService.getMessageByCode('product#UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    MessageService.getMessageByCode('product#' + messageCode).then(function(message) {
                        ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    ScreenService.showMessage(message, 'ERROR');
                }
            } else if (response && response.message) {
                ScreenService.showMessage(response.message, 'ERROR');
            } else {
                ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    var isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };

    this.changeClienteActionEventSave = function(){
        var row = widgetChangeCliente.currentRow;
        var customerPortalLogin = getLocalVar('customerPortalLogin');
        var messageParams = {
            currentCustomer: customerPortalLogin.clienteName,
            newCustomer: row.NMFANTCLIE
        };
        MessageService.confirmMessageByCode('product#ALERT_CHANGE_CUSTOMER', messageParams,
            {
                type: 'CONFIRMATION',
                success: function() {
                    ClienteService.loginUpdateSession(row.CDCLIENTE).then(function(response){
                        return widgetChangeCliente.saveRow().then(function(cliente){
                            customerPortalLogin.clienteCd = row.CDCLIENTE;
                            customerPortalLogin.clienteName = row.NMFANTCLIE;
                            customerPortalLogin.clienteRS = row.NMRAZSOCCLIE;
                            customerPortalLogin.allowProductCall = row.IDALLOWPRODUCTCAL ? row.IDALLOWPRODUCTCAL : 'N';
                            customerPortalLogin.isTeknisaClient = response.dataset.data.IS_TEKNISA_CLIENT;
                            setLocalVar('customerPortalLogin', customerPortalLogin);
                            localStorage.setItem('cdClienteLogin', row.CDCLIENTE);
                            setLocalVar('CLIENTE_NRPARCNEGOCIO', cliente.dataset.currentCliente[0].CLIENTE_NRPARCNEGOCIO);
                            setLocalVar('CLIENTE_NRORG', cliente.dataset.currentCliente[0].CLIENTE_NRORG);
                            setLocalVar('CDCLIENTE', cliente.dataset.currentCliente[0].CDCLIENTE);

                            var containers = templateManager.containers;
                            for (var key in containers) {
                                if (!containers.hasOwnProperty(key)) continue;
                                if(-1 != key.search('#')){
                                    delete containers[key];
                                }
                            }
                            
                            ScreenService.clearHistoryItems();
                            ZhFilterCreateFilterConditionsController.clearCachedFilterConditionsWidget({id: '635905682460315230345'});

                            if(templateManager.container.isCPDashboard) {
                                templateManager.container.reload();
                                templateManager.container.factoredWidgets = [];
                            } else {
                                ScreenService.openWindow('product#dashboard');
                            }
                            ScreenService.closePopup();
                            $rootScope.menuOpen = false;
                            var defaultImage = window.isCustomerPortal ? loginConfig.defaultImageProduct : loginConfig.defaultImageModules;
                            ScreenService.buildUserData(customerPortalLogin.name, [customerPortalLogin.clienteName], function() {
                                $rootScope.menuOpen = false;
                            },defaultImage);
                            ScreenService.setHeaderInfo(customerPortalLogin.clienteName);
                        });
                    }).catch(showErrorMessage);
                },
                fail: function () {}
            }
        );
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('ClienteController', ClienteController);
});
